/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/*
 * Public data structures for base64 encoding/decoding.
 */
#ifndef _NSSB64T_H_
#define _NSSB64T_H_

#include "utilrename.h"
typedef struct NSSBase64DecoderStr NSSBase64Decoder;
typedef struct NSSBase64EncoderStr NSSBase64Encoder;

#endif /* _NSSB64T_H_ */
