/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/*

  The interface to an HTTP index

*/

#include "nsISupports.idl"

interface nsIStreamListener;
interface nsIRDFDataSource;
interface nsIRDFNode;
interface nsIRDFResource;

[scriptable, uuid(6F2BDBD0-58C3-11d3-BE36-00104BDE6048)]
interface nsIHTTPIndex : nsISupports
{
    /**
     * The base URL of the HTTP index
     */
    readonly attribute string BaseURL;

    /**
     * The RDF datasource that contains the HTTP index information.
     */
    readonly attribute nsIRDFDataSource DataSource;

    /**
     * The charset to use for decoding FTP filenames
     */
    attribute string encoding;
};

%{C++

// {{2587e382-1324-11d4-a652-eadbb2be3484}
#define NS_HTTPINDEX_SERVICE_CID \
{ 0x2587e382, 0x1324, 0x11d4, { 0xa6, 0x52, 0xea, 0xdb, 0xb2, 0xbe, 0x34, 0x84 } }

#define NS_HTTPINDEX_SERVICE_CONTRACTID \
    "@mozilla.org/browser/httpindex-service;1"

#define NS_HTTPINDEX_DATASOURCE_CONTRACTID \
    "@mozilla.org/rdf/datasource;1?name=httpindex"

%}
